/*
 * XML Validation based on XML Schema (XSD)
 * 
 * by Alain VIZZINI 
 * ESSI - Sophia Antipolis
 * Microsoft France
 */

using System;
using System.IO;
using System.Xml;
using System.Xml.Schema;


namespace AlXMLValidator
{
	/// <summary>
	/// Delegate of the event that occurs every time a token is read from the XML File.
	/// </summary>
	public delegate void OnReadingEvent (String read);

	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	class MyXMLValidator
	{
		/// <summary>
		/// Event that occurs every time a token is read from the XML File.
		/// </summary>
		public event OnReadingEvent OnReadingEvent;

		/// <summary>
		/// True if some errors occured during the validation.
		/// </summary>
		private bool errors = false;
		public bool Errors{
			get {
				return errors;
			}
		}
		/// <summary>
		/// True if some warnings occured during the validation.
		/// </summary>
		private bool warnings = false;
		public bool Warnings{
			get{
				return warnings;
			}
		}


		/// <summary>
		///	The XML Schema (.xsd) file used to validate.
		/// </summary>
		private String xsdfile;
		public String XSDFile{
			get{
				return xsdfile;
			}
		}
		/// <summary>
		/// The XML file to validate.
		/// </summary>
		private String xmlfile;
		public String XMLFile{
			get
			{
				return xmlfile;
			}
		}
			
		/// <summary>
		/// Additionnal eventhandler (user provided)
		/// </summary>
		private ValidationEventHandler veh = null;


		/// <summary>
		/// Builds a new instance of MyXMLValidator
		/// </summary>
		/// <param name="XSDFile">XML Schema (.xsd) file to use to validate</param>
		/// <param name="XMLFile">XML file to validate</param>
		public MyXMLValidator (String XSDFile, String XMLFile) : this (XSDFile, XMLFile, null){}

		/// <summary>
		/// Builds a new instance of MyXMLValidator
		/// </summary>
		/// <param name="XSDFile">XML Schema (.xsd) file to use to validate</param>
		/// <param name="XMLFile">XML file to validate</param>
		/// <param name="VEH">Additional EventHandler that will be called on ValidationEvent</param>
		public MyXMLValidator (String XSDFile, String XMLFile, ValidationEventHandler VEH)
		{
			xmlfile = XMLFile;
			xsdfile = XSDFile;
			veh = VEH;
		}


		/// <summary>
		/// Checks if the file is valid (concordant with the XSD file)
		/// </summary>
		/// <remarks>Sets the Warnings and Errors attributes</remarks>
		public void Validate(){
			//create a collection of XML Schema
			XmlSchemaCollection xsdColl = new XmlSchemaCollection();
			xsdColl.Add("", new XmlTextReader(XSDFile));

			XmlValidatingReader vr = new XmlValidatingReader(new XmlTextReader (XMLFile));
			vr.ValidationType = ValidationType.Schema;
			vr.Schemas.Add(xsdColl);
			vr.ValidationEventHandler += new ValidationEventHandler (XSDValidationHandler);

			if (veh != null)
				vr.ValidationEventHandler += veh;

			while(vr.Read()) {
				OnReadingEvent (vr.Value);
			}
		}

		/// <summary>
		/// Handler called on every ValidationEvent. Used to set Errors and Warnings flags.
		/// </summary>
		/// <param name="sender">Sender of the Event</param>
		/// <param name="args">arguments</param>
		private void XSDValidationHandler(object sender, ValidationEventArgs args)
		{
			if (args.Severity == XmlSeverityType.Warning)
				warnings = true;
			if (args.Severity == XmlSeverityType.Error)
				errors = true;
		}


		/*********************************************************************************
		 * 
		 * Under this point, static test purpose methods.
		 * 
		 *********************************************************************************/
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			//
			// TODO: Add code to start application here
			//
			if (args.Length < 2)
			{
				Console.WriteLine ( "usage:\n"+
					"\tAlXMLValidator schema.xsd file.xml");				
			}
			else
			{
				String XSDFile = args[0];
				String XMLFile = args[1];

				Console.WriteLine("Starting validation");

				MyXMLValidator xv = new MyXMLValidator (XSDFile, XMLFile, new ValidationEventHandler (CustomValidationHandler));
				
				xv.OnReadingEvent += new OnReadingEvent (CustomOnReadingHandler);

				xv.Validate();

				Console.WriteLine("\nValidation finished :");
				if (xv.errors)
					Console.WriteLine ("\til y a eu des erreurs");
				if (xv.warnings)
					Console.WriteLine ("\til y a eu des warnings");
				if (!xv.errors && !xv.warnings)
					Console.WriteLine ("\t pas d'erreur ni de warnings !");
			}
		}

		public static void CustomOnReadingHandler(String s)
		{
			if (!s.Trim().Equals(""))
				Console.Write(s);
			else
				Console.Write(".");
		}

		public static void CustomValidationHandler(object sender, ValidationEventArgs args)
		{
			Console.WriteLine("***Validation error");
			Console.WriteLine("\tSeverity:{0}", args.Severity);
			Console.WriteLine("\tMessage:{0}", args.Message);
		}

	}
}
